@@ -1,6 +1,6 @@
 /*
  * ProFTPD: mod_vroot FSIO API
- * Copyright (c) 2002-2021 TJ Saunders
+ * Copyright (c) 2002-2024 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -465,7 +465,7 @@ int vroot_fsio_chroot(pr_fs_t *fs, const char *path) {
     /* Now, make sure that the given path is below the configured
      * VRootServerRoot.  If so, then we perform a real chroot to the
      * VRootServerRoot directory, then use vroots from there.
-     */ 
+     */
 
     res = strncmp(path, server_root, strlen(server_root));
 
@@ -626,7 +626,7 @@ int vroot_fsio_utimes(pr_fs_t *fs, const char *utimes_path,
   pr_pool_tag(tmp_pool, "VRoot FSIO utimes pool");
 
   path = vroot_realpath(tmp_pool, utimes_path, VROOT_REALPATH_FL_ABS_PATH);
-  
+
   if (vroot_path_lookup(NULL, vpath, sizeof(vpath)-1, path, 0, NULL) < 0) {
     xerrno = errno;
 
@@ -643,6 +643,31 @@ int vroot_fsio_utimes(pr_fs_t *fs, const char *utimes_path,
   return res;
 }
 
+const char *vroot_fsio_realpath(pr_fs_t *fs, pool *p, const char *path) {
+  const char *res = NULL;
+  int xerrno;
+  char vpath[PR_TUNABLE_PATH_MAX + 1], *real_path = NULL;
+  pool *tmp_pool = NULL;
+
+  tmp_pool = make_sub_pool(p);
+  pr_pool_tag(tmp_pool, "VRoot FSIO realpath pool");
+
+  real_path = vroot_realpath(p, path, VROOT_REALPATH_FL_ABS_PATH);
+
+  if (vroot_path_lookup(NULL, vpath, sizeof(vpath)-1, real_path, 0, NULL) < 0) {
+    xerrno = errno;
+
+    destroy_pool(tmp_pool);
+    errno = xerrno;
+    return NULL;
+  }
+
+  destroy_pool(tmp_pool);
+
+  res = pstrdup(p, vpath);
+  return res;
+}
+
 static struct dirent *vroot_dent = NULL;
 static size_t vroot_dentsz = 0;
 
@@ -834,7 +859,7 @@ void *vroot_fsio_opendir(pr_fs_t *fs, const char *orig_path) {
       /* Since this table will use DIR pointers as keys, we want to override
        * the default hashing and key comparison functions used.
        */
-    
+
       pr_table_ctl(vroot_dirtab, PR_TABLE_CTL_SET_KEY_HASH,
         vroot_dirtab_hash_cb);
       pr_table_ctl(vroot_dirtab, PR_TABLE_CTL_SET_KEY_CMP,
